@@ -36,45 +36,52 @@ from typing import List
 
 from .builders import build_trinfo
 
-if not getattr(ssl, 'HAS_SNI', False):
+if not getattr(ssl, "HAS_SNI", False):
     try:
         import urllib3.contrib.pyopenssl
+
         urllib3.contrib.pyopenssl.inject_into_urllib3()
     except ImportError:
         pass
 
-RQLIO_API = '1.0'
+RQLIO_API = "1.0"
 
 
 class SignedRequestAuth(requests.auth.AuthBase):
     """Auth implementation for CubicWeb with cube signedrequest"""
 
-    HEADERS_TO_SIGN = ('Content-MD5', 'Content-Type', 'Date')
+    HEADERS_TO_SIGN = ("Content-SHA512", "Content-Type", "Date")
 
     def __init__(self, token_id, secret):
         self.token_id = token_id
         self.secret = secret
 
     def __call__(self, req):
-        content = b''
+        content = b""
         if req.body:
             content = req.body
         if isinstance(content, str):
-            content = content.encode('utf-8')
-        req.headers['Content-MD5'] = hashlib.md5(content).hexdigest()
+            content = content.encode("utf-8")
+        req.headers["Content-SHA512"] = hashlib.sha512(content).hexdigest()
         content_to_sign = (
-            req.method + req.url
-            + ''.join(req.headers.get(field, '')
-                      for field in self.HEADERS_TO_SIGN))
-        content_signed = hmac.new(self.secret.encode('utf-8'),
-                                  content_to_sign.encode('utf-8')).hexdigest()
-        req.headers['Authorization'] = 'Cubicweb %s:%s' % (self.token_id,
-                                                           content_signed)
+            req.method
+            + req.url
+            + "".join(req.headers.get(field, "") for field in self.HEADERS_TO_SIGN)
+        )
+        content_signed = hmac.new(
+            self.secret.encode("utf-8"),
+            content_to_sign.encode("utf-8"),
+            digestmod="sha512",
+        ).hexdigest()
+        req.headers["Authorization"] = "Cubicweb %s:%s" % (
+            self.token_id,
+            content_signed,
+        )
         return req
 
 
 def date_header_value() -> str:
-    return datetime.utcnow().strftime('%a, %d %b %Y %H:%M:%S GMT')
+    return datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S GMT")
 
 
 class RemoteValidationError(Exception):
@@ -95,15 +102,15 @@ class TaskFailedError(Exception):
 
 class CWProxy(object):
     """CWProxy: A simple helper class to ease building CubicWeb_
-    clients. It allows to:
+        clients. It allows to:
 
-    * execute RQL_ queries remotely (using rqlcontroller_),
-    * access instances that requires authentication (using signedrequest_).
+        * execute RQL_ queries remotely (using rqlcontroller_),
+        * access instances that requires authentication (using signedrequest_).
 
-.. _CubicWeb: http://www.cubicweb.org/
-.. _RQL: http://docs.cubicweb.org/annexes/rql/language
-.. _rqlcontroller: http://www.cubicweb.org/project/cubicweb-rqlcontroller/
-.. _signedrequest: http://www.cubicweb.org/project/cubicweb-signedrequest/
+    .. _CubicWeb: http://www.cubicweb.org/
+    .. _RQL: http://docs.cubicweb.org/annexes/rql/language
+    .. _rqlcontroller: http://www.cubicweb.org/project/cubicweb-rqlcontroller/
+    .. _signedrequest: http://www.cubicweb.org/project/cubicweb-signedrequest/
     """
 
     def __init__(self, base_url: str, auth=None, verify=None, timeout=None):
@@ -120,18 +127,17 @@ class CWProxy(object):
         purl = urlparse.urlparse(base_url)
         # we do **not** want urls to be built with a double /, (e.g.
         # http://host// or http://host//basepath)
-        path = purl.path.strip('/')
+        path = purl.path.strip("/")
         if path:
-            path = '{:s}/{:s}'.format(purl.netloc, path)
+            path = "{:s}/{:s}".format(purl.netloc, path)
         else:
             path = purl.netloc
         self.base_url = (purl.scheme, path)
         self.auth = auth
         self.timeout = timeout
         self._ssl_verify = verify
-        self._default_vid = 'jsonexport'  # OR 'ejsonexport'?
-        self._base_url = urlparse.urlunparse(self.base_url +
-                                             ('', None, None, None))
+        self._default_vid = "jsonexport"  # OR 'ejsonexport'?
+        self._base_url = urlparse.urlunparse(self.base_url + ("", None, None, None))
 
     def handle_request(self, method: str, path: str, **kwargs):
         """Construct a requests.Request and send it through this proxy.
@@ -140,16 +146,14 @@ class CWProxy(object):
         be used to build a full URL using the base URL bound to this proxy.
         """
         msg = "handle_request() got unexpected keyword argument '{}'"
-        for unexpected in ('url', 'auth'):
+        for unexpected in ("url", "auth"):
             if unexpected in kwargs:
                 raise TypeError(msg.format(unexpected))
         url = self.build_url(path)
-        kwargs['auth'] = self.auth
-        kwargs.setdefault('headers', {}).update(
-            {'Date': date_header_value()}
-        )
-        kwargs.setdefault('verify', self._ssl_verify)
-        kwargs.setdefault('timeout', self.timeout)
+        kwargs["auth"] = self.auth
+        kwargs.setdefault("headers", {}).update({"Date": date_header_value()})
+        kwargs.setdefault("verify", self._ssl_verify)
+        kwargs.setdefault("timeout", self.timeout)
         return requests.request(method, url, **kwargs)
 
     def build_url(self, path: str, query=None):
@@ -168,7 +172,7 @@ class CWProxy(object):
         if query:
             query = urlparse.urlencode(query, doseq=True)
         if isinstance(path, (list, tuple)):
-            path = '/'.join(path)
+            path = "/".join(path)
         if path.startswith(self._base_url):
             assert query is None
             return path
@@ -180,8 +184,8 @@ class CWProxy(object):
         :param path: the path part of the URL that will be GET
         :param query: can be a sequence of two-element tuples or a doctionnary
         """
-        headers = {'Accept': 'application/json'}
-        return self.handle_request('GET', path, params=query, headers=headers)
+        headers = {"Accept": "application/json"}
+        return self.handle_request("GET", path, params=query, headers=headers)
 
     def post(self, path: str, **data):
         """Perform a POST on the cubicweb instance
@@ -190,12 +194,12 @@ class CWProxy(object):
         :param **data: will be passed as the 'data' of the request
         """
         kwargs = {
-            'headers': {'Accept': 'application/json'},
-            'data': data,
+            "headers": {"Accept": "application/json"},
+            "data": data,
         }
-        if 'files' in data:
-            kwargs['files'] = data.pop('files')
-        return self.handle_request('POST', path, **kwargs)
+        if "files" in data:
+            kwargs["files"] = data.pop("files")
+        return self.handle_request("POST", path, **kwargs)
 
     def post_json(self, path: str, payload):
         """Perform a POST on the cubicweb instance with application/json
@@ -205,10 +209,10 @@ class CWProxy(object):
         :param payload: native data to be sent as JSON (not encoded)
         """
         kwargs = {
-            'headers': {'Accept': 'application/json'},
-            'json': payload,
+            "headers": {"Accept": "application/json"},
+            "json": payload,
         }
-        return self.handle_request('POST', path, **kwargs)
+        return self.handle_request("POST", path, **kwargs)
 
     def view(self, vid: str, **args):
         """Perform a GET on <base_url>/view with <vid> and <args>
@@ -216,8 +220,8 @@ class CWProxy(object):
         :param vid: the vid of the page to retrieve
         :param **args: will be used to build the query string of the URL
         """
-        args['vid'] = vid
-        return self.get('/view', args)
+        args["vid"] = vid
+        return self.get("/view", args)
 
     def execute(self, rql: str, args=None):
         """CW connection's like execute method.
@@ -225,9 +229,20 @@ class CWProxy(object):
         :param rql: should be a unicode string or a plain ascii string
         :param args: are the optional parameters used in the query (dict)
         """
-        return self.rqlio([(rql, args or {})]).json()[0]
+        result = self.rqlio([(rql, args or {})])
 
-    def rql(self, rql: str, path='view', **data):
+        try:
+            return result.json()[0]
+        except json.decoder.JSONDecodeError as e:
+            raise Exception(
+                "Failed to code response as json. Response "
+                "(code %s) content:\n> %s\n\nException: %s"
+                % (result.status_code, result.content, e)
+            )
+
+        return result.json()[0]
+
+    def rql(self, rql: str, path="view", **data):
         """Perform an urlencoded POST to /<path> with rql=<rql>
 
         :param rql: should be a unicode string or a plain ascii string
@@ -235,32 +250,34 @@ class CWProxy(object):
         :param path: the path part of the generated URL
         :param **data: the 'data' of the request
         """
-        if rql.split()[0] in ('INSERT', 'SET', 'DELETE'):
-            raise ValueError('You must use the rqlio() method to make '
-                             'write RQL queries')
+        if rql.split()[0] in ("INSERT", "SET", "DELETE"):
+            raise ValueError(
+                "You must use the rqlio() method to make " "write RQL queries"
+            )
 
-        if not data.get('vid'):
-            data['vid'] = self._default_vid
-        if path == 'view':
-            data.setdefault('fallbackvid', '404')
+        if not data.get("vid"):
+            data["vid"] = self._default_vid
+        if path == "view":
+            data.setdefault("fallbackvid", "404")
         if rql:  # XXX may be empty?
-            if not rql.lstrip().startswith('rql:'):
+            if not rql.lstrip().startswith("rql:"):
                 # add the 'rql:' prefix to ensure given rql is considered has
                 # plain RQL so CubicWeb won't attempt other interpretation
                 # (e.g. eid, 2 or 3 word queries, plain text)
-                rql = 'rql:' + rql
-            data['rql'] = rql
+                rql = "rql:" + rql
+            data["rql"] = rql
 
         headers = {
-            'Accept': 'application/json',
-            'Date': date_header_value(),
+            "Accept": "application/json",
+            "Date": date_header_value(),
+        }
+        params = {
+            "url": self.build_url(path),
+            "headers": headers,
+            "verify": self._ssl_verify,
+            "auth": self.auth,
+            "data": data,
         }
-        params = {'url': self.build_url(path),
-                  'headers': headers,
-                  'verify': self._ssl_verify,
-                  'auth': self.auth,
-                  'data': data,
-                  }
         return requests.post(**params)
 
     def rqlio(self, queries):
@@ -284,27 +301,28 @@ class CWProxy(object):
 
         """
         headers = {
-            'Accept': 'application/json',
-            'Date': date_header_value(),
+            "Accept": "application/json",
+            "Date": date_header_value(),
         }
         files = self.preprocess_queries(queries)
 
-        params = {'url': self.build_url(('rqlio', RQLIO_API)),
-                  'headers': headers,
-                  'verify': self._ssl_verify,
-                  'auth': self.auth,
-                  'files': files,
-                  }
+        params = {
+            "url": self.build_url(("rqlio", RQLIO_API)),
+            "headers": headers,
+            "verify": self._ssl_verify,
+            "auth": self.auth,
+            "files": files,
+        }
         posted = requests.post(**params)
         if posted.status_code == 500:
             try:
                 cause = posted.json()
             except Exception as exc:
-                raise RemoteValidationError('%s (%s)', exc, posted.text)
+                raise RemoteValidationError("%s (%s)", exc, posted.text)
             else:
-                if 'reason' in cause:
+                if "reason" in cause:
                     # was a RemoteCallFailed
-                    raise RemoteValidationError(cause['reason'])
+                    raise RemoteValidationError(cause["reason"])
         return posted
 
     def preprocess_queries(self, queries):
@@ -326,24 +344,27 @@ class CWProxy(object):
             if args is None:
                 continue
             for arg_idx, (k, v) in enumerate(args.items()):
-                if hasattr(v, 'read') and callable(v.read):
+                if hasattr(v, "read") and callable(v.read):
                     # file-like object
-                    fid = args[k] = '__f%d-%d' % (query_idx, arg_idx)
+                    fid = args[k] = "__f%d-%d" % (query_idx, arg_idx)
                     files[fid] = v
                 elif isinstance(v, (date, datetime)):
                     args[k] = v.isoformat()
-        files['json'] = ('json', json.dumps(queries), 'application/json')
+        files["json"] = ("json", json.dumps(queries), "application/json")
         return files
 
-    def _set_rql_request(self, rql_request: str, kwargs, sep=',') -> str:
-        args = ['X {property_name:s} %({property_name:s})s'.format(
-            property_name=property_name) for property_name in kwargs]
+    def _set_rql_request(self, rql_request: str, kwargs, sep=",") -> str:
+        args = [
+            "X {property_name:s} %({property_name:s})s".format(
+                property_name=property_name
+            )
+            for property_name in kwargs
+        ]
         if args:
-            rql_request = u'{:s}{:s} {:s}'.format(
-                rql_request, sep, ', '.join(args))
+            rql_request = u"{:s}{:s} {:s}".format(rql_request, sep, ", ".join(args))
         return rql_request
 
-    def _rql_args_query(self, rql_request: str, kwargs, sep=',') -> list:
+    def _rql_args_query(self, rql_request: str, kwargs, sep=",") -> list:
         rql_request = self._set_rql_request(rql_request, kwargs, sep)
         response = self.rqlio([(rql_request, kwargs)])
         response.raise_for_status()
@@ -367,9 +388,9 @@ class CWProxy(object):
           1
 
         """
-        rql_query = 'Any COUNT('
-        rql_query += '1' if kwargs else 'X'
-        rql_query += ') WHERE X is {:s}'.format(entity_type)
+        rql_query = "Any COUNT("
+        rql_query += "1" if kwargs else "X"
+        rql_query += ") WHERE X is {:s}".format(entity_type)
         return int(self._rql_args_query(rql_query, kwargs)[0])
 
     def exist(self, entity_type: str, **kwargs) -> bool:
@@ -387,13 +408,13 @@ class CWProxy(object):
           False
 
         """
-        rql_query = 'Any '
-        rql_query += '1' if kwargs else 'X'
-        rql_query += ' LIMIT 1 WHERE X is {:s}'.format(entity_type)
+        rql_query = "Any "
+        rql_query += "1" if kwargs else "X"
+        rql_query += " LIMIT 1 WHERE X is {:s}".format(entity_type)
         try:
             return bool(self._rql_args_query(rql_query, kwargs))
         except RemoteValidationError as e:
-            if 'unknown entity type {:s}'.format(entity_type) in str(e):
+            if "unknown entity type {:s}".format(entity_type) in str(e):
                 return False
             raise e
 
@@ -413,8 +434,7 @@ class CWProxy(object):
           >>> self.find('CWUser', login='admin')
           [20]
         """
-        return self._rql_args_query(
-            'Any X WHERE X is {:s}'.format(entity_type), kwargs)
+        return self._rql_args_query("Any X WHERE X is {:s}".format(entity_type), kwargs)
 
     def find_one(self, entity_type: str, **kwargs) -> int:
         """Return eid of the unique entity with the given type and properties
@@ -434,11 +454,12 @@ class CWProxy(object):
           20
         """
         eids = self._rql_args_query(
-            'Any X LIMIT 2 WHERE X is {:s}'.format(entity_type), kwargs)
+            "Any X LIMIT 2 WHERE X is {:s}".format(entity_type), kwargs
+        )
         if len(eids) == 0:
-            raise NoResultError('No result for {:s}'.format(entity_type))
+            raise NoResultError("No result for {:s}".format(entity_type))
         if len(eids) > 1:
-            raise NoUniqueEntity('Cannot find unique {:s}'.format(entity_type))
+            raise NoUniqueEntity("Cannot find unique {:s}".format(entity_type))
         return eids[0]
 
     def find_last_created(self, entity_type: str, **kwargs) -> int:
@@ -460,10 +481,12 @@ class CWProxy(object):
           22
         """
         eid = self._rql_args_query(
-            'Any X ORDERBY D DESC LIMIT 1 '
-            'WHERE X is {:s}, X creation_date D'.format(entity_type), kwargs)
+            "Any X ORDERBY D DESC LIMIT 1 "
+            "WHERE X is {:s}, X creation_date D".format(entity_type),
+            kwargs,
+        )
         if not eid:
-            raise NoResultError('No result for {:s}'.format(entity_type))
+            raise NoResultError("No result for {:s}".format(entity_type))
         return eid[0]
 
     def get_state(self, eid: int) -> str:
@@ -480,18 +503,21 @@ class CWProxy(object):
           >>> self.get_state(1001)
           'wfs_finished'
         """
-        response = self.rqlio([
-            ('Any SN LIMIT 1 WHERE E eid %(eid)s, E in_state S, S name SN',
-             {'eid': eid})
-        ])
+        response = self.rqlio(
+            [
+                (
+                    "Any SN LIMIT 1 WHERE E eid %(eid)s, E in_state S, S name SN",
+                    {"eid": eid},
+                )
+            ]
+        )
         response.raise_for_status()
         rset = response.json()
         if rset and rset[0] and rset[0][0]:
             return rset[0][0][0]
         return None
 
-    def wait_for_status(self, eid: int, status,
-                        timeout=60, timesleep=1) -> None:
+    def wait_for_status(self, eid: int, status, timeout=60, timesleep=1) -> None:
         """Wait that the entity with given eid to be in status given. If it is
         not the case after timeout, a related exception is raised. The state is
         fetched and checked, then it sleeps if it is not yet the status given.
@@ -513,7 +539,7 @@ class CWProxy(object):
             current_status = self.get_state(eid)
             if current_status == status:
                 break
-            if current_status == 'wfs_failed':
+            if current_status == "wfs_failed":
                 raise TaskFailedError(eid)
             if int(time()) - start_time >= timeout:
                 raise TimeoutError(eid)
@@ -534,7 +560,7 @@ class CWProxy(object):
 
           >>> self.wait_for_finish(30)
         """
-        return self.wait_for_status(eid, 'wfs_finished', *args, **kwargs)
+        return self.wait_for_status(eid, "wfs_finished", *args, **kwargs)
 
     def change_state(self, eid: int, status: str):
         """Try to change the state with the one given for the entity that have
@@ -559,8 +585,7 @@ class CWProxy(object):
 
           >>> self.insert('Project', name='start-up nation', author='Macron')
         """
-        return self._rql_args_query(
-            'INSERT {:s} X: '.format(entity_type), kwargs, '')
+        return self._rql_args_query("INSERT {:s} X: ".format(entity_type), kwargs, "")
 
     def insert_if_not_exist(self, entity_type, **kwargs):
         """Insert an entity of the given type with given values of attributes
@@ -588,4 +613,5 @@ class CWProxy(object):
           >>> self.delete('Project', name='start-up nation', author='Macron')
         """
         return self._rql_args_query(
-            'DELETE {:s} X WHERE '.format(entity_type), kwargs, '')
+            "DELETE {:s} X WHERE ".format(entity_type), kwargs, ""
+        )
